from .graphi_types import Types
import graphene
from django.db.utils import IntegrityError

class MutationsMixin(Types, graphene.ObjectType):
    @classmethod
    def map_field_to_graphene_type(cls, field):
        field_type = field.get_internal_type()
        type_mapping = {
            'CharField': graphene.String(),
            'IntegerField': graphene.Int(),
            'BooleanField': graphene.Boolean(),
            'DateField': graphene.String(),
            'DateTimeField': graphene.String(),
            'DecimalField': graphene.Float(),
            'EmailField': graphene.String(),
            'FileField': graphene.String(),
            'FloatField': graphene.Float(),
            'ImageField': graphene.String(),
            'PositiveIntegerField': graphene.Int(),
            'PositiveSmallIntegerField': graphene.Int(),
            'SlugField': graphene.String(),
            'SmallIntegerField': graphene.Int(),
            'TextField': graphene.String(),
            'TimeField': graphene.String(),
            'UUIDField': graphene.String(),
            'BigIntegerField': graphene.Int(),
            'BinaryField': graphene.String(),
            'PositiveBigIntegerField': graphene.Int(),
            'PositiveDecimalField': graphene.Float(),
            'URLField': graphene.String(),
            'AutoField': graphene.Int(),
            'BigAutoField': graphene.Int()
        }
        return type_mapping.get(field_type)
    @classmethod
    def generate_input_type(cls, model):
        fields = cls.get_fields(model)
        return type(
            f"{model.__name__}InputType",
            (graphene.InputObjectType, ),
            {
                field: cls.map_field_to_graphene_type(model._meta.get_field(field)) for field in fields
            }
        )
    

    @classmethod
    def check_permission(user, model):
        if not user.is_authenticated:
            raise Exception('Permission Denied')
        
        if not hasattr(model, 'graphql_permissions'):
            return True
        
        if not model.graphql_permissions:
            return True
        
        if not user.has_perms(model.graphql_permissions):
            raise Exception('Permission Denied')
        
        return True





import graphene
from .mutations import MutationsMixin
from .queries import Queries


class DeleteMutation(MutationsMixin):
   
    @classmethod
    def generate_where_type(cls, model):
        return Queries.generate_where_clause(model)

    @classmethod
    def generate_argument_class(cls, model):
        where_type = cls.generate_where_type(model)
        return type(
            f'Arguments',
            (),
            {
                "where": where_type,
            }
        )

    @classmethod
    def delete_mutate(cls, model):
        def mutate(root, info, *args, **kwargs):
            cls.check_permission(info.context.user, model)
            where = kwargs.get('where')

            if not where:
                raise Exception('there is no where clause')
            
            deleted_objects = []

            queryset = model.objects.all()
            query = Queries.query_set_builder(where)
            queryset = queryset.filter(**query)
            for obj in queryset:
                obj.delete()
                deleted_objects.append(obj)

            return {
                'affected_rows': len(deleted_objects)
            }
        return mutate

    @classmethod
    def generate_delete_mutation_class(cls, model):
        argument_class = cls.generate_argument_class(model)
        _ =  type(
            f'{model.__name__}DeleteMutation',
            (graphene.Mutation,),
            {
                'Arguments': argument_class,
                'affected_rows': graphene.Int(),
                'mutate': cls.delete_mutate(model)
            }
        )
        return _

    @classmethod
    def generate_mutations(cls, _apps):
        models = cls.find_all_models(_apps)
        for model in models:
            delete_mutation_class = cls.generate_delete_mutation_class(model)
            setattr(cls, f'delete_{model.__name__.lower()}', delete_mutation_class.Field())




import graphene


class StringFilterKeywordsInputType(graphene.InputObjectType):
    exact = graphene.String()
    contains = graphene.String()
    icontains = graphene.String()
    startswith = graphene.String()
    istartswith = graphene.String()
    endswith = graphene.String()
    iendswith = graphene.String()
    regex = graphene.String()
    iregex = graphene.String()
    isnull = graphene.Boolean()


class NumberFilterKeywordsInputType(graphene.InputObjectType):
    exact = graphene.Int()
    lt = graphene.Int()
    lte = graphene.Int()
    gt = graphene.Int()
    gte = graphene.Int()
    range = graphene.List(graphene.Int)
    isnull = graphene.Boolean()


class DateFilterKeywordsInputType(graphene.InputObjectType):
    exact = graphene.String()
    lt = graphene.String()
    lte = graphene.String()
    gt = graphene.String()
    gte = graphene.String()
    range = graphene.List(graphene.String)
    isnull = graphene.Boolean()
    year = graphene.String()
    month = graphene.String()
    day = graphene.String()
    week_day = graphene.String()
    hour = graphene.String()
    minute = graphene.String()
    second = graphene.String()
    date = graphene.String()
    time = graphene.String()
    iso_year = graphene.String()
    iso_week = graphene.String()
    iso_week_day = graphene.String()

